/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.semantik.enginebased.formsolutions;

import java.util.List;
import java.util.function.Predicate;
import java.util.stream.Stream;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.IncomingFile;
import de.ozgcloud.eingang.common.formdata.IncomingFileGroup;
import de.ozgcloud.eingang.semantik.common.ZipAttachmentReader;
import de.ozgcloud.eingang.semantik.enginebased.FilesMapperHelper;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Component
@Log4j2
class FormSolutionsFilesMapper implements FormSolutionsEngineBasedMapper {

	public static final String FILE_GROUP_ZIP_NAME = "gezippte Anhänge";
	public static final String EXTRAHIERTE_ATTACHMENTS = "Extrahierte Attachments";
	public static final String ZIP_CONTENT_TYPE = "application/zip";

	@Override
	public FormData parseFormData(FormData srcFormData) {
		var formDataBuilder = srcFormData.toBuilder().clearAttachments();

		var attachments = readAttachments(srcFormData);
		if (attachments.isEmpty()) {
			return formDataBuilder.build();
		}
		return formDataBuilder.attachments(attachments).numberOfAttachments(FilesMapperHelper.countAttachedFiles(attachments)).build();
	}

	List<IncomingFileGroup> readAttachments(FormData srcFormData) {
		return new ZippedAttachmentsProcessor(srcFormData.getAttachments()).process().toList();
	}

	@RequiredArgsConstructor
	class ZippedAttachmentsProcessor {

		private final List<IncomingFileGroup> originalAttachmentsList;
		private static final Predicate<IncomingFileGroup> ZIP_FILE_GROUP = fileGroup -> StringUtils.equals(FILE_GROUP_ZIP_NAME, fileGroup.getName());

		public Stream<IncomingFileGroup> process() {
			return Stream.concat(processZipGroups(), nonZipFileGroups());
		}

		private Stream<IncomingFileGroup> processZipGroups() {
			var groupBuilder = IncomingFileGroup.builder().name("Anhänge");
			extractAttachments().forEach(groupBuilder::file);
			var group = groupBuilder.build();

			return group.getFiles().isEmpty() ? Stream.empty() : Stream.of(group);
		}

		Stream<IncomingFileGroup> nonZipFileGroups() {
			return originalAttachmentsList.stream().filter(ZIP_FILE_GROUP.negate());
		}

		Stream<IncomingFile> extractAttachments() {
			return originalAttachmentsList.stream()
					.filter(ZIP_FILE_GROUP)
					.flatMap(fileGroup -> fileGroup.getFiles().stream())
					.flatMap(this::unzip);
		}

		Stream<IncomingFile> unzip(IncomingFile zipFile) {
			try {
				return readFromZip(zipFile);
			} catch (RuntimeException e) {
				LOG.error("Cannot read source ZIP. Attach it as is.", e);
				return Stream.of(zipFile);
			}
		}

		Stream<IncomingFile> readFromZip(IncomingFile zipFile) {
			var reader = buildReader(zipFile);
			var readContent = reader.readContent();
			reader.deleteSourceFile();
			return readContent.stream();
		}

		ZipAttachmentReader buildReader(IncomingFile zipFile) {
			return ZipAttachmentReader.from(zipFile.getFile(), zipFile.getName());
		}
	}
}