package de.ozgcloud.eingang.semantik.enginebased.formcycle;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

import org.assertj.core.api.Assertions;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.eingang.common.formdata.Antragsteller;
import de.ozgcloud.eingang.common.formdata.AntragstellerTestFactory;
import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.FormDataTestFactory;

class FormcycleAntragstellerMapperTest {

	@Spy
	@InjectMocks
	private FormcycleAntragstellerMapper mapper;

	private static final String VALUE_KEY = "value";
	private static final String LABEL_KEY = "label";
	private static final Map<String, Object> ANTRAGSTELLER_DATA = new HashMap<>();
	private static final Map<String, Object> ANTRAGSTELLER_MAP = Map.of("fsBKAllDaten", Map.of(VALUE_KEY, ANTRAGSTELLER_DATA));

	@BeforeAll
	static void fillAntragstellerData() {
		ANTRAGSTELLER_DATA.put("tfAntragstellerAnrede", Map.of(VALUE_KEY, AntragstellerTestFactory.ANREDE, LABEL_KEY, "Anrede"));
		ANTRAGSTELLER_DATA.put("tfAntragstellerVorname", Map.of(LABEL_KEY, "Vorname", VALUE_KEY, AntragstellerTestFactory.VORNAME));
		ANTRAGSTELLER_DATA.put("tfAntragstellerName", Map.of(VALUE_KEY, AntragstellerTestFactory.NACHNAME, LABEL_KEY, "Nachname"));
		ANTRAGSTELLER_DATA.put("tfAntragstellerGeburtsname", Map.of(VALUE_KEY, AntragstellerTestFactory.GEBURTSNAME, LABEL_KEY, "Geburtsname"));
		ANTRAGSTELLER_DATA.put("tfAntragstellerGeburtsdatum", Map.of(VALUE_KEY, AntragstellerTestFactory.GEBURTSDATUM, LABEL_KEY, "Geburtsdatum"));
		ANTRAGSTELLER_DATA.put("tfAntragstellerGeburtsort", Map.of(VALUE_KEY, AntragstellerTestFactory.GEBURTSORT, LABEL_KEY, "Geburtsort"));
		ANTRAGSTELLER_DATA.put("tfAntragstellerEmail", Map.of(VALUE_KEY, AntragstellerTestFactory.EMAIL, LABEL_KEY, "E-Mail"));
		ANTRAGSTELLER_DATA.put("tfAntragstellerTelefon", Map.of(VALUE_KEY, AntragstellerTestFactory.TELEFON, LABEL_KEY, "Telefon"));
		ANTRAGSTELLER_DATA.put("tfAntragstellerAdresse", Map.of(LABEL_KEY, "Adresse", VALUE_KEY, AntragstellerTestFactory.STRASSE + " " + AntragstellerTestFactory.HAUSNUMMER));
		ANTRAGSTELLER_DATA.put("tfAntragstellerPLZ", Map.of(LABEL_KEY, "PLZ", VALUE_KEY, AntragstellerTestFactory.PLZ));
		ANTRAGSTELLER_DATA.put("tfAntragstellerOrt", Map.of(LABEL_KEY, "Ort", VALUE_KEY, AntragstellerTestFactory.ORT));
	}

	@Nested
	class TestParseFormData {

		@Mock
		private Antragsteller antragsteller;

		@Test
		void shouldCallGetAntragsteller() {
			mapper.parseFormData(buildFormData());

			verify(mapper).getAntragstellerData(ANTRAGSTELLER_MAP);
		}

		@Test
		void shouldCallBuildAntragsteller() {
			doReturn(Optional.of(ANTRAGSTELLER_DATA)).when(mapper).getAntragstellerData(anyMap());

			mapper.parseFormData(buildFormData());

			verify(mapper).buildAntragsteller(ANTRAGSTELLER_DATA);
		}

		@Test
		void shouldSetAntragsteller() {
			doReturn(antragsteller).when(mapper).buildAntragsteller(anyMap());

			var formData = mapper.parseFormData(buildFormData());

			assertThat(formData.getAntragsteller()).isSameAs(antragsteller);
		}

		@Test
		void shouldReturnSameFormData() {
			var formData = FormDataTestFactory.create();

			var result = mapper.parseFormData(formData);

			assertThat(result).isSameAs(formData);
		}

		FormData buildFormData() {
			return FormData.builder().formData(ANTRAGSTELLER_MAP).build();
		}
	}

	@Nested
	class TestGetAntragstellerData {

		@Test
		void shouldReturnEmpty() {
			Map<String, Object> formDataMap = Map.of("key", "value");

			var result = mapper.getAntragstellerData(formDataMap);

			assertThat(result).isEmpty();
		}

		@Test
		void shouldReturnAntragstellerData() {
			var result = mapper.getAntragstellerData(ANTRAGSTELLER_MAP);

			assertThat(result).contains(ANTRAGSTELLER_DATA);
		}
	}

	@Nested
	class TestBuildAntragsteller {

		@Test
		void shouldSetAnrede() {
			var result = buildAntragsteller();

			assertThat(result.getAnrede()).isEqualTo(AntragstellerTestFactory.ANREDE);
		}

		@Test
		void shouldSetVorname() {
			var result = buildAntragsteller();

			assertThat(result.getVorname()).isEqualTo(AntragstellerTestFactory.VORNAME);
		}

		@Test
		void shouldSetNachname() {
			var result = buildAntragsteller();

			assertThat(result.getNachname()).isEqualTo(AntragstellerTestFactory.NACHNAME);
		}

		@Test
		void shouldSetGeburtsname() {
			var result = buildAntragsteller();

			assertThat(result.getGeburtsname()).isEqualTo(AntragstellerTestFactory.GEBURTSNAME);
		}

		@Test
		void shouldSetGeburtsdatum() {
			var result = buildAntragsteller();

			assertThat(result.getGeburtsdatum()).isEqualTo(AntragstellerTestFactory.GEBURTSDATUM);
		}

		@Test
		void shouldSetGeburtsort() {
			var result = buildAntragsteller();

			assertThat(result.getGeburtsort()).isEqualTo(AntragstellerTestFactory.GEBURTSORT);
		}

		@Test
		void shouldSetEmail() {
			var result = buildAntragsteller();

			assertThat(result.getEmail()).isEqualTo(AntragstellerTestFactory.EMAIL);
		}

		@Test
		void shouldSetTelefon() {
			var result = buildAntragsteller();

			assertThat(result.getTelefon()).isEqualTo(AntragstellerTestFactory.TELEFON);
		}

		@Test
		void shouldSetAdresse() {
			var result = buildAntragsteller();

			assertThat(result.getStrasse()).isEqualTo(AntragstellerTestFactory.STRASSE + " " + AntragstellerTestFactory.HAUSNUMMER);
		}

		@Test
		void shouldSetPlz() {
			var result = buildAntragsteller();

			assertThat(result.getPlz()).isEqualTo(AntragstellerTestFactory.PLZ);
		}

		@Test
		void shouldSetOrt() {
			var result = buildAntragsteller();

			assertThat(result.getOrt()).isEqualTo(AntragstellerTestFactory.ORT);
		}

		Antragsteller buildAntragsteller() {
			return mapper.buildAntragsteller(ANTRAGSTELLER_DATA);
		}
	}
}