package de.ozgcloud.eingang.semantik.enginebased.xta;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.io.InputStream;
import java.util.List;
import java.util.stream.IntStream;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.MockedStatic;
import org.mockito.Mockito;
import org.mockito.Spy;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.IncomingFile;
import de.ozgcloud.eingang.common.formdata.IncomingFileTestFactory;
import de.ozgcloud.eingang.semantik.common.ZipAttachmentReader;
import lombok.SneakyThrows;

class XtaZipRepresentationsMapperTest {

	private static final String ZIP_CONTENT_TYPE = "application/zip";

	@Spy
	private final XtaZipRepresentationsMapper mapper = new XtaZipRepresentationsMapper();

	@Nested
	class TestExtractZipFileEctract {

		@Test
		void shouldExtractZipFiles() {
			try (var zipAttachment = Mockito.mockStatic(ZipAttachmentReader.class)) {
				initZipRepresentationMocks(zipAttachment);

				var formData = mapper.parseFormData(createTestFormDataWithZipRepresentation());

				assertThat(formData.getRepresentations()).hasSize(3);
			}
		}

		FormData createTestFormDataWithZipRepresentation() {
			List<IncomingFile> representations = List.of(
					IncomingFileTestFactory.createBuilder()
							.name("attachments.zip")
							.contentType(ZIP_CONTENT_TYPE)
							.build());

			return FormData.builder().representations(representations).numberOfRepresentations(representations.size()).build();
		}

		@Test
		void shouldSetRepresentationsNumber() {
			try (var zipAttachment = Mockito.mockStatic(ZipAttachmentReader.class)) {
				initZipRepresentationMocks(zipAttachment);

				var formData = mapper.parseFormData(createTestFormDataWithZipRepresentation());

				assertThat(formData.getNumberOfRepresentations()).isEqualTo(3);
			}
		}

		@Test
		void shouldIgnoreNonZipFiles() {
			try (var zipAttachment = Mockito.mockStatic(ZipAttachmentReader.class)) {

				var formData = mapper.parseFormData(createTestFormDataWithoutZipRepresentation());

				assertThat(formData.getNumberOfRepresentations()).isEqualTo(1);
			}
		}

		FormData createTestFormDataWithoutZipRepresentation() {
			List<IncomingFile> representations = List.of(
					IncomingFileTestFactory.create());
			return FormData.builder().representations(representations).numberOfRepresentations(representations.size()).build();
		}

		@Test
		void shouldNotDeleteOriginalZipFile() {
			try (var zipAttachment = Mockito.mockStatic(ZipAttachmentReader.class)) {
				initZipRepresentationMocks(zipAttachment);

				var formData = mapper.parseFormData(createTestFormDataWithZipRepresentation());

				assertThat(formData.getRepresentations()).map(IncomingFile::getContentType)
						.filteredOn(e -> e.equals(XtaZipRepresentationsMapper.ZIP_CONTENT_TYPE)).hasSize(1);
			}
		}

		@Test
		void shouldDoNothingOnEmptyRepresentations() {
			try (var zipAttachment = Mockito.mockStatic(ZipAttachmentReader.class)) {

				var formData = mapper.parseFormData(createTestFormDataWithoutRepresentation());

				assertThat(formData.getRepresentations()).isEmpty();
			}
		}

		FormData createTestFormDataWithoutRepresentation() {
			return FormData.builder().numberOfRepresentations(0).build();
		}
	}

	@Test
	void testIsZipFilePredicate() {

		assertThat(XtaZipRepresentationsMapper.IS_ZIP_FILE.test(IncomingFileTestFactory.create())).isFalse();
		assertThat(XtaZipRepresentationsMapper.IS_ZIP_FILE.test(IncomingFileTestFactory.createBuilder().contentType(ZIP_CONTENT_TYPE).build()))
				.isTrue();
	}

	@SneakyThrows
	private static void initZipRepresentationMocks(MockedStatic<ZipAttachmentReader> zipAttachmentMock) {
		var contentEntries = IntStream.range(0, 2).boxed().map(i -> IncomingFileTestFactory.createBuilder().name(i.toString()).build()).toList();
		ZipAttachmentReader mock = when(mock(ZipAttachmentReader.class).readContent()).thenReturn(contentEntries).getMock();
		zipAttachmentMock.when(() -> ZipAttachmentReader.from(any(InputStream.class), any())).thenReturn(mock);
	}
}
